@@ -1,22 +1,22 @@
 /* Test file for in-place operations.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -25,7 +25,7 @@ MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
 
-#define DISP(s, t) {printf(s); mpfr_out_str(stdout, 10, 0, t, GMP_RNDN); }
+#define DISP(s, t) {printf(s); mpfr_out_str(stdout, 2, 0, t, GMP_RNDN); }
 #define DISP2(s,t) {DISP(s,t); putchar('\n');}
 
 #define SPECIAL_MAX 12
@@ -75,7 +75,7 @@ set_special (mpfr_ptr x, unsigned int select)
       MPFR_SET_EXP (x, MPFR_GET_EXP (x)-1);
       break;
     default:
-      mpfr_random (x);
+      mpfr_urandomb (x, RANDS);
       break;
     }
 }
@@ -481,7 +481,8 @@ test3a (int (*testfunc)(mpfr_ptr, mpfr_ptr, mpfr_srcptr, mp_rnd_t),
       testfunc (res1, res2, res1, rnd);
       if (mpfr_compare (res1, ref1) || mpfr_compare (res2, ref2))
         {
-          printf ("Error for %s(a, b, a) for ", foo);
+          printf ("Error for %s(a, b, a) for rnd=%s, ", foo,
+                  mpfr_print_rnd_mode (rnd));
           DISP2("a=",ref3);
           DISP("expected (", ref1); DISP(",",ref2);
           DISP("), got (", res1); DISP(",", res2); printf(")\n");
@@ -518,92 +519,113 @@ reldiff_wrapper (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 int
 main (void)
 {
-  mp_rnd_t rnd;
+  int rnd;
   mp_prec_t p;
-  MPFR_TEST_USE_RANDS ();
   tests_start_mpfr ();
 
   p = (randlimb () % 200)+ MPFR_PREC_MIN;
   RND_LOOP (rnd)
-    {
-      test3 (mpfr_add, "mpfr_add", p, rnd);
-      test3 (mpfr_sub, "mpfr_sub", p, rnd);
-      test3 (mpfr_mul, "mpfr_mul", p, rnd);
-      test3 (mpfr_div, "mpfr_div", p, rnd);
+  {
+    test2a (mpfr_round, "mpfr_round", p);
+    test2a (mpfr_ceil, "mpfr_ceil", p);
+    test2a (mpfr_floor, "mpfr_floor", p);
+    test2a (mpfr_trunc, "mpfr_trunc", p);
 
-      test3 (mpfr_agm, "mpfr_agm", p, rnd);
-      test3 (mpfr_min, "mpfr_min", p, rnd);
-      test3 (mpfr_max, "mpfr_max", p, rnd);
-      test3 (reldiff_wrapper, "mpfr_reldiff", p, rnd);
+    test2ui (mpfr_add_ui, "mpfr_add_ui", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_div_2exp, "mpfr_div_2exp", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_div_ui, "mpfr_div_ui", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_mul_2exp, "mpfr_mul_2exp", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_mul_ui, "mpfr_mul_ui", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_pow_ui, "mpfr_pow_ui", p, (mp_rnd_t) rnd);
+    test2ui (mpfr_sub_ui, "mpfr_sub_ui", p, (mp_rnd_t) rnd);
 
-      test2a (mpfr_round, "mpfr_round", p);
-      test2a (mpfr_ceil, "mpfr_ceil", p);
-      test2a (mpfr_floor, "mpfr_floor", p);
-      test2a (mpfr_trunc, "mpfr_trunc", p);
+    testui2 (mpfr_ui_div, "mpfr_ui_div", p, (mp_rnd_t) rnd);
+    testui2 (mpfr_ui_sub, "mpfr_ui_sub", p, (mp_rnd_t) rnd);
+    testui2 (mpfr_ui_pow, "mpfr_ui_pow", p, (mp_rnd_t) rnd);
 
-      test2ui (mpfr_add_ui, "mpfr_add_ui", p, rnd);
-      test2ui (mpfr_mul_2exp, "mpfr_mul_2exp", p, rnd);
-      test2ui (mpfr_mul_ui, "mpfr_mul_ui", p, rnd);
-      test2ui (mpfr_pow_ui, "mpfr_pow_ui", p, rnd);
-      test2ui (mpfr_sub_ui, "mpfr_sub_ui", p, rnd);
-      test2ui (mpfr_div_2exp, "mpfr_div_2exp", p, rnd);
-      test2ui (mpfr_div_ui, "mpfr_div_ui", p, rnd);
+    test2 (mpfr_sqr, "mpfr_sqr", p, (mp_rnd_t) rnd);
+    test2 (mpfr_sqrt, "mpfr_sqrt", p, (mp_rnd_t) rnd);
+    test2 (mpfr_abs, "mpfr_abs", p, (mp_rnd_t) rnd);
+    test2 (mpfr_neg, "mpfr_neg", p, (mp_rnd_t) rnd);
 
-      testui2 (mpfr_ui_div, "mpfr_ui_div", p, rnd);
-      testui2 (mpfr_ui_sub, "mpfr_ui_sub", p, rnd);
-      testui2 (mpfr_ui_pow, "mpfr_ui_pow", p, rnd);
+    test2 (mpfr_log, "mpfr_log", p, (mp_rnd_t) rnd);
+    test2 (mpfr_log2, "mpfr_log2", p, (mp_rnd_t) rnd);
+    test2 (mpfr_log10, "mpfr_log10", p, (mp_rnd_t) rnd);
+    test2 (mpfr_log1p, "mpfr_log1p", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_sqr, "mpfr_sqr", p, rnd);
-      test2 (mpfr_sqrt, "mpfr_sqrt", p, rnd);
-      test2 (mpfr_abs, "mpfr_abs", p, rnd);
-      test2 (mpfr_neg, "mpfr_neg", p, rnd);
+    test2 (mpfr_exp, "mpfr_exp", p, (mp_rnd_t) rnd);
+    test2 (mpfr_exp2, "mpfr_exp2", p, (mp_rnd_t) rnd);
+    test2 (mpfr_exp10, "mpfr_exp10", p, (mp_rnd_t) rnd);
+    test2 (mpfr_expm1, "mpfr_expm1", p, (mp_rnd_t) rnd);
+    test2 (mpfr_eint, "mpfr_eint", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_log, "mpfr_log", p, rnd);
-      test2 (mpfr_log2, "mpfr_log2", p, rnd);
-      test2 (mpfr_log10, "mpfr_log10", p, rnd);
-      test2 (mpfr_log1p, "mpfr_log1p", p, rnd);
+    test2 (mpfr_sinh, "mpfr_sinh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_cosh, "mpfr_cosh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_tanh, "mpfr_tanh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_asinh, "mpfr_asinh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_acosh, "mpfr_acosh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_atanh, "mpfr_atanh", p, (mp_rnd_t) rnd);
+    test2 (mpfr_sech, "mpfr_sech", p, (mp_rnd_t) rnd);
+    test2 (mpfr_csch, "mpfr_csch", p, (mp_rnd_t) rnd);
+    test2 (mpfr_coth, "mpfr_coth", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_exp, "mpfr_exp", p, rnd);
-      test2 (mpfr_exp2, "mpfr_exp2", p, rnd);
-      test2 (mpfr_exp10, "mpfr_exp10", p, rnd);
-      test2 (mpfr_expm1, "mpfr_expm1", p, rnd);
-      test2 (mpfr_eint, "mpfr_eint", p, rnd);
+    test2 (mpfr_asin, "mpfr_asin", p, (mp_rnd_t) rnd);
+    test2 (mpfr_acos, "mpfr_acos", p, (mp_rnd_t) rnd);
+    test2 (mpfr_atan, "mpfr_atan", p, (mp_rnd_t) rnd);
+    test2 (mpfr_cos, "mpfr_cos", p, (mp_rnd_t) rnd);
+    test2 (mpfr_sin, "mpfr_sin", p, (mp_rnd_t) rnd);
+    test2 (mpfr_tan, "mpfr_tan", p, (mp_rnd_t) rnd);
+    test2 (mpfr_sec, "mpfr_sec", p, (mp_rnd_t) rnd);
+    test2 (mpfr_csc, "mpfr_csc", p, (mp_rnd_t) rnd);
+    test2 (mpfr_cot, "mpfr_cot", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_sinh, "mpfr_sinh", p, rnd);
-      test2 (mpfr_cosh, "mpfr_cosh", p, rnd);
-      test2 (mpfr_tanh, "mpfr_tanh", p, rnd);
-      test2 (mpfr_asinh, "mpfr_asinh", p, rnd);
-      test2 (mpfr_acosh, "mpfr_acosh", p, rnd);
-      test2 (mpfr_atanh, "mpfr_atanh", p, rnd);
+    test2 (mpfr_erf,  "mpfr_erf",  p, (mp_rnd_t) rnd);
+    test2 (mpfr_erfc, "mpfr_erfc", p, (mp_rnd_t) rnd);
+    test2 (mpfr_j0,   "mpfr_j0",   p, (mp_rnd_t) rnd);
+    test2 (mpfr_j1,   "mpfr_j1",   p, (mp_rnd_t) rnd);
+    test2 (mpfr_y0,   "mpfr_y0",   p, (mp_rnd_t) rnd);
+    test2 (mpfr_y1,   "mpfr_y1",   p, (mp_rnd_t) rnd);
+    test2 (mpfr_zeta, "mpfr_zeta", p, (mp_rnd_t) rnd);
+    test2 (mpfr_gamma, "mpfr_gamma", p, (mp_rnd_t) rnd);
+    test2 (mpfr_lngamma, "mpfr_lngamma", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_asin, "mpfr_asin", p, rnd);
-      test2 (mpfr_acos, "mpfr_acos", p, rnd);
-      test2 (mpfr_atan, "mpfr_atan", p, rnd);
-      test2 (mpfr_cos, "mpfr_cos", p, rnd);
-      test2 (mpfr_sin, "mpfr_sin", p, rnd);
-      test2 (mpfr_tan, "mpfr_tan", p, rnd);
+    test2 (mpfr_rint, "mpfr_rint", p, (mp_rnd_t) rnd);
+    test2 (mpfr_rint_ceil, "mpfr_rint_ceil", p, (mp_rnd_t) rnd);
+    test2 (mpfr_rint_floor, "mpfr_rint_floor", p, (mp_rnd_t) rnd);
+    test2 (mpfr_rint_round, "mpfr_rint_round", p, (mp_rnd_t) rnd);
+    test2 (mpfr_rint_trunc, "mpfr_rint_trunc", p, (mp_rnd_t) rnd);
+    test2 (mpfr_frac, "mpfr_frac", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_sec, "mpfr_sec", p, rnd);
-      test2 (mpfr_csc, "mpfr_csc", p, rnd);
-      test2 (mpfr_cot, "mpfr_cot", p, rnd);
+    test3 (mpfr_add, "mpfr_add", p, (mp_rnd_t) rnd);
+    test3 (mpfr_sub, "mpfr_sub", p, (mp_rnd_t) rnd);
+    test3 (mpfr_mul, "mpfr_mul", p, (mp_rnd_t) rnd);
+    test3 (mpfr_div, "mpfr_div", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_erf,  "mpfr_erf",  p, rnd);
-      test2 (mpfr_erfc, "mpfr_erfc", p, rnd);
-      test2 (mpfr_j0,   "mpfr_j0",   p, rnd);
-      test2 (mpfr_j1,   "mpfr_j1",   p, rnd);
+    test3 (mpfr_agm, "mpfr_agm", p, (mp_rnd_t) rnd);
+    test3 (mpfr_min, "mpfr_min", p, (mp_rnd_t) rnd);
+    test3 (mpfr_max, "mpfr_max", p, (mp_rnd_t) rnd);
 
-      test2 (mpfr_zeta, "mpfr_zeta", p, rnd);
-      test2 (mpfr_gamma, "mpfr_gamma", p, rnd);
-      test2 (mpfr_rint, "mpfr_rint", p, rnd);
-      test2 (mpfr_frac, "mpfr_frac", p, rnd);
+    test3 (reldiff_wrapper, "mpfr_reldiff", p, (mp_rnd_t) rnd);
+    test3 (mpfr_dim, "mpfr_dim", p, (mp_rnd_t) rnd);
 
-      test3 (mpfr_pow, "mpfr_pow", p, rnd);
-      test3 (mpfr_atan2, "mpfr_atan2", p, rnd);
+    test3 (mpfr_remainder, "mpfr_remainder", p, (mp_rnd_t) rnd);
+    test3 (mpfr_pow, "mpfr_pow", p, (mp_rnd_t) rnd);
+    test3 (mpfr_atan2, "mpfr_atan2", p, (mp_rnd_t) rnd);
+    test3 (mpfr_hypot, "mpfr_hypot", p, (mp_rnd_t) rnd);
 
-      test4 (mpfr_fma, "mpfr_fma", p, rnd);
+    test3a (mpfr_sin_cos, "mpfr_sin_cos", p, (mp_rnd_t) rnd);
 
-      test3a (mpfr_sin_cos, "mpfr_sin_cos", p, rnd);
-    }
+    test4 (mpfr_fma, "mpfr_fma", p, (mp_rnd_t) rnd);
+    test4 (mpfr_fms, "mpfr_fms", p, (mp_rnd_t) rnd);
+
+#if MPFR_VERSION >= MPFR_VERSION_NUM(2,4,0)
+    test2 (mpfr_li2, "mpfr_li2",  p, (mp_rnd_t) rnd);
+    test2 (mpfr_rec_sqrt, "mpfr_rec_sqrt",  p, (mp_rnd_t) rnd);
+    test3 (mpfr_fmod, "mpfr_fmod", p, (mp_rnd_t) rnd);
+    test3a (mpfr_modf, "mpfr_modf", p, (mp_rnd_t) rnd);
+    test3a (mpfr_sinh_cosh, "mpfr_sinh_cosh", p, (mp_rnd_t) rnd);
+#endif
+  }
 
   tests_end_mpfr ();
   return 0;
