@@ -1,34 +1,58 @@
-/* Generic test file for functions with one or two mpfr_t arguments.
+/* Generic test file for functions with one or two arguments (the second being
+   either mpfr_t or double).
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
-/* define TWO_ARGS for two-argument functions like mpfr_pow */
+/* define TWO_ARGS for two-argument functions like mpfr_pow
+   define DOUBLE_ARG1 or DOUBLE_ARG2 for function with a double operand in
+   first or second place like sub_d or d_sub */
 
+#ifndef TEST_RANDOM_POS
+/* For the random function: one number on two is negative. */
+#define TEST_RANDOM_POS 256
+#endif
+
+#ifndef TEST_RANDOM_POS2
+/* For the random function: one number on two is negative. */
+#define TEST_RANDOM_POS2 256
+#endif
+
+#ifndef TEST_RANDOM_EMIN
+#define TEST_RANDOM_EMIN -256
+#endif
+
+#ifndef TEST_RANDOM_EMAX
+#define TEST_RANDOM_EMAX 255
+#endif
+
+/* The (void *) below is needed to avoid a warning with gcc 4.2+ and functions
+ * with 2 arguments. See <http://gcc.gnu.org/bugzilla/show_bug.cgi?id=36299>.
+ */
 #define TGENERIC_FAIL(S, X, U)                                          \
   do                                                                    \
     {                                                                   \
       printf ("%s\nx = ", (S));                                         \
       mpfr_out_str (stdout, 2, 0, (X), GMP_RNDN);                       \
       printf ("\n");                                                    \
-      if (U)                                                            \
+      if ((void *) U != 0)                                              \
         {                                                               \
           printf ("u = ");                                              \
           mpfr_out_str (stdout, 2, 0, (U), GMP_RNDN);                   \
@@ -42,7 +66,7 @@ MA 02110-1301, USA. */
   while (0)
 
 #undef TGENERIC_CHECK
-#if defined(TWO_ARGS)
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
 #define TGENERIC_CHECK(S, EXPR) \
   do if (!(EXPR)) TGENERIC_FAIL (S, x, u); while (0)
 #else
@@ -50,7 +74,7 @@ MA 02110-1301, USA. */
   do if (!(EXPR)) TGENERIC_FAIL (S, x, 0); while (0)
 #endif
 
-#if DEBUG_TGENERIC
+#ifdef DEBUG_TGENERIC
 #define STR(F) #F
 #define TGENERIC_IAUX(F,P,X,U)                                          \
   do                                                                    \
@@ -69,7 +93,7 @@ MA 02110-1301, USA. */
     }                                                                   \
   while (0)
 #undef TGENERIC_INFO
-#if defined(TWO_ARGS)
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
 #define TGENERIC_INFO(F,P) TGENERIC_IAUX(F,P,x,u)
 #else
 #define TGENERIC_INFO(F,P) TGENERIC_IAUX(F,P,x,0)
@@ -90,16 +114,24 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
   mpfr_t x, y, z, t;
 #ifdef TWO_ARGS
   mpfr_t u;
+#elif defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+  mpfr_t u;
+  double d;
 #endif
   mp_rnd_t rnd;
   int inexact, compare, compare2;
   unsigned int n;
+  unsigned long ctrt = 0, ctrn = 0;
+  mp_exp_t old_emin, old_emax;
+
+  old_emin = mpfr_get_emin ();
+  old_emax = mpfr_get_emax ();
 
   mpfr_init (x);
   mpfr_init (y);
   mpfr_init (z);
   mpfr_init (t);
-#ifdef TWO_ARGS
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
   mpfr_init (u);
 #endif
 
@@ -124,57 +156,73 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
           mpfr_set_prec (x, xprec);
 #ifdef TWO_ARGS
           mpfr_set_prec (u, xprec);
+#elif defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+          mpfr_set_prec (u, IEEE_DBL_MANT_DIG);
 #endif
 
           if (n > 3 || prec < p1)
             {
 #if defined(RAND_FUNCTION)
               RAND_FUNCTION (x);
-#ifdef TWO_ARGS
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
               RAND_FUNCTION (u);
 #endif
 #else
-              tests_default_random (x);
-#ifdef TWO_ARGS
-              tests_default_random (u);
+              tests_default_random (x, TEST_RANDOM_POS,
+                                    TEST_RANDOM_EMIN, TEST_RANDOM_EMAX);
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+              tests_default_random (u, TEST_RANDOM_POS2,
+                                    TEST_RANDOM_EMIN, TEST_RANDOM_EMAX);
 #endif
 #endif
             }
-          else if (n <= 1)
+          else
             {
-              /* Special cases tested in precision p1 if n <= 1. */
-              mpfr_set_si (x, n == 0 ? 1 : -1, GMP_RNDN);
-              mpfr_set_exp (x, mpfr_get_emin ());
-#ifdef TWO_ARGS
-              mpfr_set_si (u, randlimb () % 2 == 0 ? 1 : -1, GMP_RNDN);
-              mpfr_set_exp (u, mpfr_get_emin ());
+              /* Special cases tested in precision p1 if n <= 3. They are
+                 useful really in the extended exponent range. */
+              set_emin (MPFR_EMIN_MIN);
+              set_emax (MPFR_EMAX_MAX);
+              if (n <= 1)
+                {
+                  mpfr_set_si (x, n == 0 ? 1 : -1, GMP_RNDN);
+                  mpfr_set_exp (x, mpfr_get_emin ());
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+                  mpfr_set_si (u, randlimb () % 2 == 0 ? 1 : -1, GMP_RNDN);
+                  mpfr_set_exp (u, mpfr_get_emin ());
 #endif
-            }
-          else  /* 2 <= n <= 3 */
-            {
-              /* Special cases tested in precision p1 if 2 <= n <= 3. */
-              if (getenv ("MPFR_CHECK_MAX") == NULL)
-                continue;
-              mpfr_set_si (x, n == 0 ? 1 : -1, GMP_RNDN);
-              mpfr_setmax (x, REDUCE_EMAX);
-#ifdef TWO_ARGS
-              mpfr_set_si (u, randlimb () % 2 == 0 ? 1 : -1, GMP_RNDN);
-              mpfr_setmax (u, mpfr_get_emax ());
+                }
+              else  /* 2 <= n <= 3 */
+                {
+                  if (getenv ("MPFR_CHECK_MAX") == NULL)
+                    goto next_n;
+                  mpfr_set_si (x, n == 0 ? 1 : -1, GMP_RNDN);
+                  mpfr_setmax (x, REDUCE_EMAX);
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+                  mpfr_set_si (u, randlimb () % 2 == 0 ? 1 : -1, GMP_RNDN);
+                  mpfr_setmax (u, mpfr_get_emax ());
 #endif
+                }
             }
 
-          rnd = (mp_rnd_t) RND_RAND ();
+          rnd = RND_RAND ();
           mpfr_clear_flags ();
 #ifdef DEBUG_TGENERIC
           TGENERIC_INFO (TEST_FUNCTION, MPFR_PREC (y));
 #endif
 #if defined(TWO_ARGS)
           compare = TEST_FUNCTION (y, x, u, rnd);
+#elif defined(DOUBLE_ARG1)
+          d = mpfr_get_d (u, rnd);
+          compare = TEST_FUNCTION (y, d, x, rnd);
+#elif defined(DOUBLE_ARG2)
+          d = mpfr_get_d (u, rnd);
+          compare = TEST_FUNCTION (y, x, d, rnd);
 #else
           compare = TEST_FUNCTION (y, x, rnd);
 #endif
           TGENERIC_CHECK ("Bad inexact flag",
                           (compare != 0) ^ (mpfr_inexflag_p () == 0));
+          ctrt++;
           if (MPFR_IS_SINGULAR (y))
             {
               if (MPFR_IS_NAN (y) || mpfr_nanflag_p ())
@@ -203,6 +251,7 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
             }
           else if (mpfr_can_round (y, yprec, rnd, rnd, prec))
             {
+              ctrn++;
               mpfr_set (t, y, rnd);
               /* Risk of failures are known when some flags are already set
                  before the function call. Do not set the erange flag, as
@@ -216,11 +265,15 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
 #endif
 #if defined(TWO_ARGS)
               inexact = TEST_FUNCTION (z, x, u, rnd);
+#elif defined(DOUBLE_ARG1)
+              inexact = TEST_FUNCTION (z, d, x, rnd);
+#elif defined(DOUBLE_ARG2)
+              inexact = TEST_FUNCTION (z, x, d, rnd);
 #else
               inexact = TEST_FUNCTION (z, x, rnd);
 #endif
               if (mpfr_erangeflag_p ())
-                continue;
+                goto next_n;
               if (mpfr_nan_p (z) || mpfr_cmp (t, z) != 0)
                 {
                   printf ("results differ for x=");
@@ -228,6 +281,9 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
 #ifdef TWO_ARGS
                   printf ("\nu=");
                   mpfr_out_str (stdout, 2, xprec, u, GMP_RNDN);
+#elif defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+                  printf ("\nu=");
+                  mpfr_out_str (stdout, 2, IEEE_DBL_MANT_DIG, u, GMP_RNDN);
 #endif
                   printf (" prec=%u rnd_mode=%s\n", (unsigned) prec,
                           mpfr_print_rnd_mode (rnd));
@@ -256,7 +312,7 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
                   printf ("Wrong inexact flag for rnd=%s: expected %d, got %d"
                           "\n", mpfr_print_rnd_mode (rnd), compare, inexact);
                   printf ("x="); mpfr_print_binary (x); puts ("");
-#ifdef TWO_ARGS
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
                   printf ("u="); mpfr_print_binary (u); puts ("");
 #endif
                   printf ("y="); mpfr_print_binary (y); puts ("");
@@ -264,14 +320,46 @@ test_generic (mp_prec_t p0, mp_prec_t p1, unsigned int N)
                   exit (1);
                 }
             }
+          else if (getenv ("MPFR_SUSPICIOUS_OVERFLOW") != NULL)
+            {
+              /* For developers only! */
+              MPFR_ASSERTN (MPFR_IS_PURE_FP (y));
+              mpfr_nexttoinf (y);
+              if (MPFR_IS_INF (y) && MPFR_IS_LIKE_RNDZ (rnd, MPFR_IS_NEG (y))
+                  && !mpfr_overflow_p ())
+                {
+                  printf ("Possible bug! |y| is the maximum finite number "
+                          "and has been obtained when\nrounding toward zero"
+                          " (%s). Thus there is a very probable overflow,\n"
+                          "but the overflow flag is not set!\n",
+                          mpfr_print_rnd_mode (rnd));
+                  printf ("x="); mpfr_print_binary (x); puts ("");
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
+                  printf ("u="); mpfr_print_binary (u); puts ("");
+#endif
+                  exit (1);
+                }
+            }
+
+        next_n:
+          /* In case the exponent range has been changed by
+             tests_default_random() or for special values... */
+          mpfr_set_emin (old_emin);
+          mpfr_set_emax (old_emax);
         }
     }
 
+#ifndef TGENERIC_NOWARNING
+  if (3 * ctrn < 2 * ctrt)
+    printf ("Warning! Too few normal cases in generic tests (%lu / %lu)\n",
+            ctrn, ctrt);
+#endif
+
   mpfr_clear (x);
   mpfr_clear (y);
   mpfr_clear (z);
   mpfr_clear (t);
-#ifdef TWO_ARGS
+#if defined(TWO_ARGS) || defined(DOUBLE_ARG1) || defined(DOUBLE_ARG2)
   mpfr_clear (u);
 #endif
 }
